*-------------------------------------------------------------------------------------
* Private Debt Overhang and the Government Spending Multiplier
* forthcoming in the Journal of Applied Econometrics
* by Marco Bernardini and Gert Peersman (June 2017)
*
* This file refers to the transmission mechanism (DN shocks)
*-------------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
* 1) SETTINGS
////////////////////////////////////////////////////////////////////////////////
****************************
* housekeeping and loading *
****************************

clear all // clear memory 
cls // clear results window
set more off // do not display the "more" message
set matsize 300 // maximum number of variables in the model
pause off // allow for pauses
adopath ++ PLUS

global expe "DN_transmission" // name of the experiment

global path F:\Research\Projects\BP_LP\JAE\

use "${path}data\data_20170711.dta", clear

tsset quarter, q // time variable and data frequency

capture log close
log using "${path}log files\log_${expe}.log", replace 

********************************************
* set parameters that govern specification *
********************************************

local p=4   //lag order
global hor=12-1 // maximum horizon (h=0,...,hor)
global band 90 // 68 (1SD), 90 (btw 1 and 2 SD), 95 (2SD)

global exerc_mar sdA sdB // states of the economy
global begsample=1919 // beginning of the sample (1919q1 earliest)
global endsample=2013 // end of the sample (2013q4 latest)

***************************
* construct raw variables *
***************************

drop if  quarter<tq(${begsample}q1) // define the final sample (start) before defining variables
drop if  quarter>tq(${endsample}q4) // define the final sample (end) before defining variables





* Raw variables
gen gdp=RGDP
gen gov=RGCE
gen con=RCON
gen inv=RINV
gen nr=NINT/100
gen tx=AMTR/100
gen pd=PRY/100
gen gd=GVY/100
gen dn=DFNW/100
gen df=log(DEF)
gen rd=pd*RGDP

*****************************
* construct final variables *
*****************************

* LHS variables and reference for RHS lagged controls
foreach var in gov gdp con inv rd { // compute var_h for h=0,...,H
	forvalues j=0/$hor {
		gen `var'x`j' = (F`j'.`var'-L.`var')/L.gdp // DIFF(t+h - t-1) scaled by predet GDP (t-1)
	}
}

foreach var in tx df nr pd gd { // compute var_h for h=0,...,H
	forvalues j=0/$hor {
		gen `var'x`j' = (F`j'.`var'-L.`var') // DIFF(t+h - t-1)
	}
}

foreach var in dn { // compute var_h for h=0,...,H
	forvalues j=0/$hor {
		gen `var'x`j' = F`j'.`var' // LEV(t+h)
	}
}

foreach var in gov nr tx pd gd df rd { // cumulative LHS variables
	qui gen cum_`var'=0
	forvalues i=0/$hor {
		gen cum`i'_`var'=`var'x`i'+cum_`var'
		replace cum_`var'=cum`i'_`var'
	}
	drop cum_`var'
}

gen sample=.
replace sample=1 if govx$hor & gdpx$hor !=. & L`p'.gdpx0!=. & L`p'.govx0!=. // estimation sample (fixed along the horizon H)
gen shock=dn // Ramey shocks

gen t = _n // count observation by ID
gen h=t-1 // horizon starting a 0

gen nonlin=L.PRO






* define states
gen sdA = (1-nonlin)			// outside private debt overhang
gen sdB = nonlin				// inside private debt overhang


*select the states
local totstat sdA sdB // analyzed states (shocks and/or controls)
local ctrs sdB // do not include sdA to avoid multicollinearity with the constant term

* RHS variables
foreach state in `totstat' {
	* shock
	gen shock_`state'=shock*`state' // shocks
	* lagged controls
	forvalues j=1/`p' { // lagged controls
		foreach ctr in gov gdp con inv gd pd nr tx df dn { // also defense news among controls
			gen l`j'`ctr'x0=L`j'.`ctr'x0
			gen l`j'`ctr'x0_`state'=l`j'`ctr'x0*`state'
			drop l`j'`ctr'x0
			local ctrs `ctrs' l`j'`ctr'x0_`state'
		}
	}
}

* Prestorage PEs and SEs
foreach inc in gov nr tx pd gd df rd {
	foreach ex_m in $exerc_mar sdBA {
		* impulse responses
		qui gen b`ex_m'_`inc'=.
		qui gen se`ex_m'_`inc'=.
		foreach cint in $band {
			qui gen lo`ex_m'_`inc'_`cint'=.
			qui gen up`ex_m'_`inc'_`cint'=.
		}
		* cumulative multipliers (1-step)
		qui gen bmul`ex_m'_`inc'=.
		qui gen semul`ex_m'_`inc'=.
		foreach cint in $band {
			qui gen lomul`ex_m'_`inc'_`cint'=.
			qui gen upmul`ex_m'_`inc'_`cint'=.
		}
		* cumulative multipliers (3-steps)
		qui gen bmul`ex_m'3s_`inc'=0
		qui gen bmul`ex_m'3s_`inc'temp=0
		qui gen bmul`ex_m'3s_`inc'gov=.
	}
}

////////////////////////////////////////////////////////////////////////////////
* 2) 1-STEP ESTIMATION
////////////////////////////////////////////////////////////////////////////////

foreach inc in gov nr tx pd gd df rd {
	forvalues i=0/$hor {
		if "`inc'"!="gov" { // skip the multiplier gov over gov
			capture drop cumfit_gov_sdA cumfit_gov_sdB
			local ord=`i'+1 // ord='i'+1 in ivreg2 is equivalent to ord='i' in xtscc (check "help ivreg2")
			gen cumfit_gov_sdA = cum`i'_gov*sdA
			gen cumfit_gov_sdB = cum`i'_gov*sdB
			
			
			* calculate state-dependent multipliers (main equation)
			ivreg2 `inc'x`i' (cumfit_gov_sdA cumfit_gov_sdB = shock_sdA shock_sdB) `ctrs' if sample==1, robust bw(auto) noid
			qui gen bmulsdA`i' = _b[cumfit_gov_sdA]
			qui gen semulsdA`i'= _se[cumfit_gov_sdA]
			qui gen bmulsdB`i' = _b[cumfit_gov_sdB]
			qui gen semulsdB`i'= _se[cumfit_gov_sdB]
			lincom cumfit_gov_sdB - cumfit_gov_sdA
			qui gen bmulsdBA`i' = r(estimate)
			qui gen semulsdBA`i'= r(se)
			estadd	scalar diffBA = r(estimate)
			estadd	scalar diffBAse = r(se)
			test cumfit_gov_sdA = cumfit_gov_sdB
			estadd	scalar diffBApv = r(p)
			local kpar = e(df_m)+1
			estadd scalar par = `kpar'
			eststo tabout1s_`i', title("main_`i'") // store regression output
			* calculate state-dependent F-stats and weakiv robust p-value for the difference (only for GDP)
			if "`inc'"=="gdp" {
				ivreg2 cum`i'_`inc' (cumfit_gov_sdA = shock_sdA) `ctrs' if sample==1, robust bw(auto)
				estadd scalar FsdA = e(widstat)
				eststo taboutFA_`i', title("sdA_`i'") // store regression output
				*
				ivreg2 cum`i'_`inc' (cumfit_gov_sdB = shock_sdB) `ctrs' if sample==1, robust bw(auto)
				estadd scalar FsdB = e(widstat)
				eststo taboutFB_`i', title("sdB_`i'") // store regression output
				*
				
				
				
				
					weakiv ivreg2 cum`i'_`inc' (cum`i'_gov cumfit_gov_sdB = shock shock_sdB) `ctrs' if sample==1, robust bw(auto) strong(cum`i'_gov)
					estadd scalar diffBApvar = e(ar_p) // conditional on cum`i'_gov being a strong instrument (see RZ17)
				
				eststo tabout1sdiffAR_`i', title("diffAR_`i'") // store regression output
			}
			* storing
			foreach ex_m in $exerc_mar sdBA {
				qui replace bmul`ex_m'_`inc'=bmul`ex_m'`i' if h==`i'
				qui replace semul`ex_m'_`inc'=semul`ex_m'`i' if h==`i'
				foreach cint in $band {
					qui replace upmul`ex_m'_`inc'_`cint'=bmul`ex_m'_`inc'+invnormal((100+`cint')/2/100)*semul`ex_m'_`inc' if h==`i'
					qui replace lomul`ex_m'_`inc'_`cint'=bmul`ex_m'_`inc'-invnormal((100+`cint')/2/100)*semul`ex_m'_`inc' if h==`i'
				}
			}

			if (`i'==0 | `i'==3 |`i'==7 |`i'==11) & "`inc'"=="gdp" {
				local tabout_`inc' `tabout_`inc'' taboutFA_`i' taboutFB_`i' tabout1sdiffAR_`i' tabout1s_`i'
			}
			else if  (`i'==0 | `i'==3 |`i'==7 |`i'==11) & "`inc'"!="gdp" {
				local tabout_`inc' `tabout_`inc'' tabout1s_`i'
			}
			drop bmul*`i' semul*`i'
		}

////////////////////////////////////////////////////////////////////////////////
* 3) 3-STEPS ESTIMATION
////////////////////////////////////////////////////////////////////////////////

		ivreg2 `inc'x`i' shock_sdA shock_sdB `ctrs' if sample==1, robust bw(auto) //  choose btw "auto" or "`ord'"
		
		* individual responses
		gen b`inc'h`i'_sdA=_b[shock_sdA]
		gen se`inc'h`i'_sdA=_se[shock_sdA]
		gen b`inc'h`i'_sdB=_b[shock_sdB]
		gen se`inc'h`i'_sdB=_se[shock_sdB]
		
		
		
		
		
		* cumulative responses
		foreach ex_m in $exerc_mar {
			qui gen bmul`ex_m'3s_`inc'h`i'=b`inc'h`i'_`ex_m'+bmul`ex_m'3s_`inc'temp
			drop bmul`ex_m'3s_`inc'temp
			qui gen bmul`ex_m'3s_`inc'temp=bmul`ex_m'3s_`inc'h`i'
			qui replace b`ex_m'_`inc'=b`inc'h`i'_`ex_m' if h==`i'
			qui replace se`ex_m'_`inc'=se`inc'h`i'_`ex_m' if h==`i'
			foreach cint in $band {
				qui replace up`ex_m'_`inc'_`cint'=b`ex_m'_`inc'+invnormal((100+`cint')/2/100)*se`ex_m'_`inc' if h==`i'
				qui replace lo`ex_m'_`inc'_`cint'=b`ex_m'_`inc'-invnormal((100+`cint')/2/100)*se`ex_m'_`inc' if h==`i'
			}
			qui replace bmul`ex_m'3s_`inc'=bmul`ex_m'3s_`inc'h`i' if h==`i'
			replace bmul`ex_m'3s_`inc'gov=b`ex_m'_`inc'/bmul`ex_m'3s_gov if h==`i' // ratio between IRF and cumulative responses of government spending




			sum bmul`ex_m'3s_`inc'gov if h==`i'
			estadd scalar `ex_m'3s = r(mean)
		}
	
		eststo	tabout3s_`i', title("3S_`i'") // store regression output
		if `i'==0 | `i'==3 |`i'==7 |`i'==11 {
			local tabout_`inc' `tabout_`inc'' tabout3s_`i'
		}
	}
	
	////////////////////////////////////////////////////////////////////////////////
	* 4) Output
	////////////////////////////////////////////////////////////////////////////////
	
	** Regression table
	if "`inc'"=="gdp" {
	esttab `tabout_`inc'' using "${path}results\tab_`inc'_${expe}.csv", mtitle ///
	replace page se nonum b(2) se(2) sfmt(2) obslast label nonotes ///
	keep(cumfit_gov_sdA cumfit_gov_sdB) ///
	nostar scalars("diffBA" "diffBAse" "diffBApv" "diffBApvar" "sdA3s" "sdB3s" "par Parameters" "FsdA" "FsdB")
	local tabout
	}
	if "`inc'"!="gov" &  "`inc'"!="gdp" {
	esttab `tabout_`inc'' using "${path}results\tab_`inc'_${expe}.csv", mtitle ///
	replace page se nonum b(2) se(2) sfmt(2) obslast label nonotes ///
	keep(cumfit_gov_sdA cumfit_gov_sdB) ///
	nostar scalars("diffBA" "diffBAse" "diffBApv" "sdA3s" "sdB3s" "par Parameters")
	local tabout
	}
}
** Data for figures (MATLAB)
export excel h	bmulsdA_nr		upmulsdA_nr_90		lomulsdA_nr_90		///
				bmulsdB_nr		upmulsdB_nr_90		lomulsdB_nr_90		///
				bmulsdBA_nr		upmulsdBA_nr_90		lomulsdBA_nr_90		///
				bmulsdA_tx		upmulsdA_tx_90		lomulsdA_tx_90		///
				bmulsdB_tx		upmulsdB_tx_90		lomulsdB_tx_90		///
				bmulsdBA_tx		upmulsdBA_tx_90		lomulsdBA_tx_90		///
				bmulsdA_pd		upmulsdA_pd_90		lomulsdA_pd_90		///
				bmulsdB_pd		upmulsdB_pd_90		lomulsdB_pd_90		///
				bmulsdBA_pd		upmulsdBA_pd_90		lomulsdBA_pd_90		///
				bmulsdA_gd		upmulsdA_gd_90		lomulsdA_gd_90		///
				bmulsdB_gd		upmulsdB_gd_90		lomulsdB_gd_90		///
				bmulsdBA_gd		upmulsdBA_gd_90		lomulsdBA_gd_90		///
				bmulsdA_df		upmulsdA_df_90		lomulsdA_df_90		///
				bmulsdB_df		upmulsdB_df_90		lomulsdB_df_90		///
				bmulsdBA_df		upmulsdBA_df_90		lomulsdBA_df_90		///
				bmulsdA_rd		upmulsdA_rd_90		lomulsdA_rd_90		///
				bmulsdB_rd		upmulsdB_rd_90		lomulsdB_rd_90		///
				bmulsdBA_rd		upmulsdBA_rd_90		lomulsdBA_rd_90		///
				if h<=$hor using "${path}article\figurestables\figMATLAB.xls", sheet("Fig6_DN_data",replace) firstrow(var)
log close
