*-------------------------------------------------------------------------------------
* Private Debt Overhang and the Government Spending Multiplier
* forthcoming in the Journal of Applied Econometrics
* by Marco Bernardini and Gert Peersman (June 2017)
*
* This file generates the augmented estimation (DN shocks)
*-------------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
* 1) SETTINGS
////////////////////////////////////////////////////////////////////////////////
****************************
* housekeeping and loading *
****************************

clear all // clear memory 
cls // clear results window
set more off // do not display the "more" message
set matsize 300 // maximum number of variables in the model
pause off // allow for pauses
adopath ++ PLUS

global expe "DN_augmented" // name of the experiment

global path F:\Research\Projects\BP_LP\JAE\

use "${path}data\data_20170711.dta", clear

tsset quarter, q // time variable and data frequency

capture log close
log using "${path}log files\log_${expe}.log", replace 

********************************************
* set parameters that govern specification *
********************************************

local p=4   //lag order
global hor=12-1 // maximum horizon (h=0,...,hor)
global band 90 // 68 (1SD), 90 (btw 1 and 2 SD), 95 (2SD)

global exerc_mar sdA sdB sdC // states of the economy
global begsample=1919 // beginning of the sample (1919q1 earliest)
global endsample=2013 // end of the sample (2013q4 latest)

***************************
* construct raw variables *
***************************

drop if  quarter<tq(${begsample}q1) // define the final sample (start) before defining variables
drop if  quarter>tq(${endsample}q4) // define the final sample (end) before defining variables





* Raw variables
gen gdp=RGDP
gen gov=RGCE
gen con=RCON
gen inv=RINV
gen nr=NINT/100
gen tx=AMTR/100
gen pd=PRY/100
gen gd=GVY/100
gen dn=DFNW/100
gen df=log(DEF)
gen rd=pd*RGDP

*****************************
* construct final variables *
*****************************

* LHS variables and reference for RHS lagged controls
foreach var in gov gdp con inv rd { // compute var_h for h=0,...,H
	forvalues j=0/$hor {
		gen `var'x`j' = (F`j'.`var'-L.`var')/L.gdp // DIFF(t+h - t-1) scaled by predet GDP (t-1)
	}
}

foreach var in tx df nr pd gd { // compute var_h for h=0,...,H
	forvalues j=0/$hor {
		gen `var'x`j' = (F`j'.`var'-L.`var') // DIFF(t+h - t-1)
	}
}

foreach var in dn { // compute var_h for h=0,...,H
	forvalues j=0/$hor {
		gen `var'x`j' = F`j'.`var' // LEV(t+h)
	}
}

foreach var in gov gdp { // cumulative LHS variables
	qui gen cum_`var'=0
	forvalues i=0/$hor {
		gen cum`i'_`var'=`var'x`i'+cum_`var'
		replace cum_`var'=cum`i'_`var'
	}
	drop cum_`var'
}

gen sample=.
replace sample=1 if govx$hor & gdpx$hor !=. & L`p'.gdpx0!=. & L`p'.govx0!=. // estimation sample (fixed along the horizon H)
gen shock=dn // Ramey shocks

gen t = _n // count observation by ID
gen h=t-1 // horizon starting a 0

gen nonlinA=L.PRO

foreach aug in DELEV SMC BC GVO AGV UNS RECQ ZLB {
preserve

gen nonlinB=L.`aug'

*define states
gen sdA = 1						// neutral state
gen sdB = nonlinA				// augmented private debt overhang
gen sdC = nonlinB				// augmented  alternative

*select the states
local totstat sdA sdB sdC // analyzed states (shocks and/or controls)
local ctrs sdB sdC // do not include sdA to avoid multicollinearity with the constant term

* RHS variables
foreach state in `totstat' {
	* shock
	gen shock_`state'=shock*`state' // shocks
	* lagged controls
	forvalues j=1/`p' { // lagged controls
		foreach ctr in gov gdp con inv gd pd nr tx df dn { // also defense news among controls
			gen l`j'`ctr'x0=L`j'.`ctr'x0
			gen l`j'`ctr'x0_`state'=l`j'`ctr'x0*`state'
			drop l`j'`ctr'x0
			local ctrs `ctrs' l`j'`ctr'x0_`state'
		}
	}
}

* Prestorage PEs and SEs
foreach inc in gov gdp {
	foreach ex_m in $exerc_mar {
		* impulse responses
		qui gen b`ex_m'_`inc'=.
		qui gen se`ex_m'_`inc'=.
		foreach cint in $band {
			qui gen lo`ex_m'_`inc'_`cint'=.
			qui gen up`ex_m'_`inc'_`cint'=.
		}
		* cumulative multipliers (1-step)
		qui gen bmul`ex_m'_`inc'=.
		qui gen semul`ex_m'_`inc'=.
		foreach cint in $band {
			qui gen lomul`ex_m'_`inc'_`cint'=.
			qui gen upmul`ex_m'_`inc'_`cint'=.
		}
		* cumulative multipliers (3-steps)
		qui gen bmul`ex_m'3s_`inc'=0
		qui gen bmul`ex_m'3s_`inc'temp=0
		qui gen bmul`ex_m'3s_`inc'gov=.
	}
}

////////////////////////////////////////////////////////////////////////////////
* 2) 1-STEP ESTIMATION
////////////////////////////////////////////////////////////////////////////////

foreach inc in gov gdp {
	forvalues i=0/$hor {
		if "`inc'"!="gov" { // skip the multiplier gov over gov
			capture drop cumfit_gov_sdA cumfit_gov_sdB cumfit_gov_sdC
			local ord=`i'+1 // ord='i'+1 in ivreg2 is equivalent to ord='i' in xtscc (check "help ivreg2")
			gen cumfit_gov_sdA = cum`i'_gov*sdA
			gen cumfit_gov_sdB = cum`i'_gov*sdB
			gen cumfit_gov_sdC = cum`i'_gov*sdC

			* calculate state-dependent multipliers (main equation)
			ivreg2 cum`i'_`inc' (cumfit_gov_sdA cumfit_gov_sdB cumfit_gov_sdC = shock_sdA shock_sdB shock_sdC) `ctrs' if sample==1, robust bw(auto) noid
			qui gen bmulsdA`i' = _b[cumfit_gov_sdA]
			qui gen semulsdA`i'= _se[cumfit_gov_sdA]
			qui gen bmulsdB`i' = _b[cumfit_gov_sdB]
			qui gen semulsdB`i'= _se[cumfit_gov_sdB]
			qui gen bmulsdC`i' = _b[cumfit_gov_sdC]
			qui gen semulsdC`i'= _se[cumfit_gov_sdC]
			
			
			
			
			
			local kpar = e(df_m)+1
			estadd scalar par = `kpar'
			eststo tabout1s_`i', title("main_`i'") // store regression output
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			* storing
			foreach ex_m in $exerc_mar {
				qui replace bmul`ex_m'_`inc'=bmul`ex_m'`i' if h==`i'
				qui replace semul`ex_m'_`inc'=semul`ex_m'`i' if h==`i'
				foreach cint in $band {
					qui replace upmul`ex_m'_`inc'_`cint'=bmul`ex_m'_`inc'+invnormal((100+`cint')/2/100)*semul`ex_m'_`inc' if h==`i'
					qui replace lomul`ex_m'_`inc'_`cint'=bmul`ex_m'_`inc'-invnormal((100+`cint')/2/100)*semul`ex_m'_`inc' if h==`i'
				}
			}
			
			
			
			
			if `i'==0 | `i'==3 |`i'==7 |`i'==11 {
				local tabout_`inc'_`aug' `tabout_`inc'_`aug'' tabout1s_`i'
			}
			drop bmul*`i' semul*`i'
		}

////////////////////////////////////////////////////////////////////////////////
* 3) 3-STEPS ESTIMATION
////////////////////////////////////////////////////////////////////////////////

		ivreg2 `inc'x`i' shock_sdA shock_sdB shock_sdC `ctrs' if sample==1, robust bw(auto) //  choose btw "auto" or "`ord'"
		
		* individual responses
		gen b`inc'h`i'_sdA=_b[shock_sdA]
		gen se`inc'h`i'_sdA=_se[shock_sdA]
		lincom shock_sdA + shock_sdB
		gen b`inc'h`i'_sdB=r(estimate)
		gen se`inc'h`i'_sdB=r(se)
		lincom shock_sdA + shock_sdC
		gen b`inc'h`i'_sdC=r(estimate)
		gen se`inc'h`i'_sdC=r(se)
		
		* cumulative responses
		foreach ex_m in $exerc_mar {
			qui gen bmul`ex_m'3s_`inc'h`i'=b`inc'h`i'_`ex_m'+bmul`ex_m'3s_`inc'temp
			drop bmul`ex_m'3s_`inc'temp
			qui gen bmul`ex_m'3s_`inc'temp=bmul`ex_m'3s_`inc'h`i'
			qui replace b`ex_m'_`inc'=b`inc'h`i'_`ex_m' if h==`i'
			qui replace se`ex_m'_`inc'=se`inc'h`i'_`ex_m' if h==`i'
			foreach cint in $band {
				qui replace up`ex_m'_`inc'_`cint'=b`ex_m'_`inc'+invnormal((100+`cint')/2/100)*se`ex_m'_`inc' if h==`i'
				qui replace lo`ex_m'_`inc'_`cint'=b`ex_m'_`inc'-invnormal((100+`cint')/2/100)*se`ex_m'_`inc' if h==`i'
			}
			qui replace bmul`ex_m'3s_`inc'=bmul`ex_m'3s_`inc'h`i' if h==`i'
			replace bmul`ex_m'3s_`inc'gov=bmul`ex_m'3s_`inc'/bmul`ex_m'3s_gov if h==`i' // ratio between cumulative responses
			if "`ex_m'"!="sdA" {
				replace bmul`ex_m'3s_`inc'gov=bmul`ex_m'3s_`inc'gov-bmulsdA3s_`inc'gov if h==`i' // ratio between cumulative responses
			}
			
			sum bmul`ex_m'3s_`inc'gov if h==`i'
			estadd scalar `ex_m'3s = r(mean)
		}
	
		eststo	tabout3s_`i', title("3S_`i'") // store regression output
		if `i'==0 | `i'==3 |`i'==7 |`i'==11 {
			local tabout_`inc'_`aug' `tabout_`inc'_`aug'' tabout3s_`i'
		}
	}
	
	////////////////////////////////////////////////////////////////////////////////
	* 4) Output
	////////////////////////////////////////////////////////////////////////////////
	
	** Regression table
	
	
	
	
	
	
	
	if "`inc'"!="gov" {
	esttab `tabout_`inc'_`aug'' using "${path}results\tab_`inc'_${expe}_`aug'.csv", mtitle ///
	replace page se nonum b(2) se(2) sfmt(2) obslast label nonotes ///
	keep(cumfit_gov_sdA cumfit_gov_sdB cumfit_gov_sdC) ///
	nostar scalars("sdA3s" "sdB3s" "sdC3s" "par Parameters")
	local tabout
	}
}
restore
}
log close
